<?php

namespace Drupal\commerce_ticketing\Mail;

use Drupal\commerce\MailHandlerInterface;
use Drupal\commerce_ticketing\CommerceTicketInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;

class TicketReceiptMail {

  use StringTranslationTrait;

  /**
   * The mail handler.
   *
   * @var \Drupal\commerce\MailHandlerInterface
   */
  protected $mailHandler;

  /**
   * The module handler to invoke the alter hook.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * TicketReceiptMail constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\commerce\MailHandlerInterface $mail_handler
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, MailHandlerInterface $mail_handler, ModuleHandlerInterface $module_handler) {
    $this->mailHandler = $mail_handler;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public function send(CommerceTicketInterface $ticket, $to = NULL, $bcc = NULL) {

    $order = $ticket->getOrder();

    $to = isset($to) ? $to : $order->getEmail();
    if (!$to) {
      // The email should not be empty.
      return FALSE;
    }

    $subject = $this->t('Ticket for your order #@number', ['@number' => $order->getOrderNumber()]);
    $body = [
      '#theme' => 'commerce_ticket_receipt',
      '#order_entity' => $order,
      '#ticket' => $ticket,

    ];

    $params = [
      'id' => 'ticket_receipt',
      'from' => $order->getStore()->getEmail(),
      'bcc' => $bcc,
      'order' => $order,
    ];

    if ($this->moduleHandler->moduleExists('commerce_ticketing_pdf')) {
      $pdf_url = Url::fromRoute('commerce_ticketing_pdf.download_pdf', ['ticket' => $ticket->uuid()], ['absolute' => TRUE]);
      $params['files'][] = (object) [
        'filename' => "ticket.pdf",
        'uri' => $pdf_url->toString(),
        'filemime' => 'application/pdf',
      ];
    }

    $customer = $order->getCustomer();
    if ($customer->isAuthenticated()) {
      $params['langcode'] = $customer->getPreferredLangcode();
    }

    // Allow other modules to alter the ticket receipt mail.
    $mail_data = ['body' => $body, 'params' => $params];
    $this->moduleHandler->alter('ticket_receipt_mail', $mail_data, $ticket);
    $body = $mail_data['body'];
    $params = $mail_data['params'];

    return $this->mailHandler->sendMail($to, $subject, $body, $params);
  }

}
